/* Bluetooth stack headers */
#include "bg_types.h"
#include "native_gecko.h"
#include "gatt_db.h"
#include "em_cmu.h"
#include "app.h"

#define TIMER_ID	1

/* Local function prototypes */
static uint32_t makeRandomPasskey(void);
static uint32_t readPasskey(void);

/* Flag for indicating DFU Reset must be performed */
static uint8_t boot_to_dfu = 0;
static uint32_t passkey;
static uint8_t bonding;
static uint8_t *addr, conn_handle;
static uint8_t counter;
static char c;

/* Main application */
void appMain(gecko_configuration_t *pconfig)
{
#if DISABLE_SLEEP > 0
  pconfig->sleep.flags = 0;
#endif

  GPIO_PinModeSet(BSP_LED0_PORT, BSP_LED0_PIN, gpioModePushPull, 0);
  GPIO_PinModeSet(BSP_LED1_PORT, BSP_LED1_PIN, gpioModePushPull, 0);
  GPIO_PinModeSet(BSP_BUTTON0_PORT, BSP_BUTTON0_PIN, gpioModeInputPull, 1);
  GPIO_PinModeSet(BSP_BUTTON1_PORT, BSP_BUTTON1_PIN, gpioModeInputPull, 1);

  /* Initialize debug prints. Note: debug prints are off by default. See DEBUG_LEVEL in app.h */
  initLog();

  /* Initialize stack */
  gecko_init(pconfig);

  while (1) {
    /* Event pointer for handling events */
    struct gecko_cmd_packet* evt;

    /* if there are no events pending then the next call to gecko_wait_event() may cause
     * device go to deep sleep. Make sure that debug prints are flushed before going to sleep */
    if (!gecko_event_pending()) {
      flushLog();
    }

    /* Check for stack event. This is a blocking event listener. If you want non-blocking please see UG136. */
    evt = gecko_wait_event();

    /* Handle events */
    switch (BGLIB_MSG_ID(evt->header)) {
      case gecko_evt_system_boot_id:										// Bluetooth stack is ready
    	  printf("\r\n");
    	  if (GPIO_PinInGet(BSP_BUTTON0_PORT, BSP_BUTTON0_PIN)){			// BUTTON0 is not pressed
    		  printf("Connecting with bonded devices only\r\n");
    		  gecko_cmd_sm_configure(0x17, sm_io_capability_displayonly);	// connect with bonded devices only
    		  gecko_cmd_sm_set_bondable_mode(0);							// disable new bondings
    	  }
    	  else {															// BUTTON0 is pressed: enable new bondings
    		  printf("All connections are enabled. Release BUTTON0...\r\n");
//    		  gecko_cmd_sm_configure(0x07, sm_io_capability_displayonly);	// passkey displayed by phone (initiator)
    		  gecko_cmd_sm_configure(0x07, sm_io_capability_keyboardonly);	// passkey displayed by this app (responder)
//    		  gecko_cmd_sm_configure(0x07, sm_io_capability_keyboarddisplay); // numeric comparison mode
    		  gecko_cmd_sm_set_bondable_mode(1);							// enable new bondings
        	  passkey = makeRandomPasskey();								// set passkey to be displayed
        	  gecko_cmd_sm_set_passkey(passkey);							//    with displayonly I/O capability
    	  }

    	  if (!GPIO_PinInGet(BSP_BUTTON1_PORT, BSP_BUTTON1_PIN)) {			// erase all bondings
    		  gecko_cmd_sm_delete_bondings();								//    if BUTTON1 is pressed
    		  printf("All bondings are erased. Release BUTTON1...\r\n");
    	  }

		  printf("Existing bondings:\r\n");
    	  gecko_cmd_sm_list_all_bondings();									// list all existing bondings
      break;

      case gecko_evt_le_connection_opened_id:								// connection is opened
    	  conn_handle = evt->data.evt_le_connection_opened.connection;
    	  GPIO_PinOutSet(BSP_LED0_PORT, BSP_LED0_PIN);						// turn on red LED
    	  printLog("connection opened\r\n");
//    	  gecko_sm_increase_security();										// in our case it is optional
      break;

      case gecko_evt_le_connection_closed_id:								// connection closed
    	  GPIO_PinOutClear(BSP_LED0_PORT, BSP_LED0_PIN);					// turn off red LED
    	  printLog("connection closed, reason: 0x%2.2x\r\n\r\n", evt->data.evt_le_connection_closed.reason);

    	  if (boot_to_dfu) {												// Check if need to boot to OTA DFU mode
    		  gecko_cmd_system_reset(2);									// Enter to OTA DFU mode
    	  } else {
    		  gecko_cmd_le_gap_start_advertising(0, le_gap_general_discoverable, le_gap_connectable_scannable);
    	  }
      break;

      case gecko_evt_gatt_server_user_write_request_id:						// some characteristic is updated
    	  if (evt->data.evt_gatt_server_user_write_request.characteristic == gattdb_ota_control) {
    		  boot_to_dfu = 1;												// Set flag to enter to OTA mode
    		  gecko_cmd_gatt_server_send_user_write_response(				// send response to Write Request
    				  evt->data.evt_gatt_server_user_write_request.connection,
					  gattdb_ota_control, bg_err_success);
    		  /* Close connection to enter to DFU OTA mode */
    		  gecko_cmd_le_connection_close(evt->data.evt_gatt_server_user_write_request.connection);
    	  }
      break;

      case gecko_evt_hardware_soft_timer_id:								// software timer event
    	  counter++;
    	  gecko_cmd_gatt_server_write_attribute_value(gattdb_counter_value, 0, 1, (const uint8*)&counter);
      break;

      /***************** Security related events ******************/
      case gecko_evt_sm_list_bonding_entry_id:								// next existing bonding record
    	  bonding = evt->data.evt_sm_list_bonding_entry.bonding;			// bonding index
    	  addr = evt->data.evt_sm_list_bonding_entry.address.addr;			// bonded address
    	  printf("%d: %02x:%02x:%02x:%02x:%02x:%02x \r\n", bonding,			// print all the above
    			  addr[5], addr[4], addr[3], addr[2], addr[1], addr[0]);
      break;

      case gecko_evt_sm_list_all_bondings_complete_id:						// end of existing bondings list
    	  printf("End of list\r\n");
    	  printf("starting advertisements\r\n");							// just start advertisements
    	  gecko_cmd_le_gap_set_advertise_timing(0, 160, 160, 0, 0);			// set 100ms advert. period
          gecko_cmd_le_gap_start_advertising(0, le_gap_general_discoverable, le_gap_connectable_scannable);
          gecko_cmd_hardware_set_soft_timer(32768,TIMER_ID,0);				// counter update timer
      break;

      case gecko_evt_sm_passkey_display_id:									// passkey for establishing new bonding
     	  printf("Type in this passkey on phone: %d\r\n", (int)evt->data.evt_sm_passkey_display.passkey);
      break;

      case gecko_evt_sm_passkey_request_id:									// enter passkey displayed on phone
    	  printf("Enter the passkey displayed on phone: ");
    	  passkey = readPasskey();											// get user input (passkey)
    	  printf("\r\nEntered passkey: %d\r\n", (int)passkey);
    	  gecko_cmd_sm_enter_passkey(conn_handle, passkey);					// set entered passkey
	  break;

      case gecko_evt_sm_confirm_bonding_id:									// new bonding request is received
     	  gecko_cmd_sm_bonding_confirm(conn_handle, 1);						// accept bonding request
     	  printf("bonding request accepted\r\n");
      break;

      case gecko_evt_sm_bonded_id:											// new bonding outcome (success)
    	  printf("bond success\r\n");
      break;

      case gecko_evt_sm_bonding_failed_id:									// new bonding outcome (failure)
          printf("bonding failed, reason 0x%2X\r\n", evt->data.evt_sm_bonding_failed.reason);
          gecko_cmd_le_connection_close(evt->data.evt_sm_bonding_failed.connection);
      break;

      case gecko_evt_sm_confirm_passkey_id:									// used in numeric comparison mode
    	  printf("Do you see this passkey on the other device: %06u? (y/n)  ",
    			  (int)evt->data.evt_sm_confirm_passkey.passkey);
    	  do {c = getchar();} while (c == 255);								// wait for key pressed
    	  printf("%c\r\n", c);												// echo the user input
    	  if (c == 'y' || c == 'Y') {
    		  gecko_cmd_sm_passkey_confirm(conn_handle, 1);					// confirm passkey match
    		  printf("Waiting for other device to confirm...\r\n");
    	  }
    	  else {
    		  gecko_cmd_sm_passkey_confirm(conn_handle, 0);					// do not confirm passkey match
          }
      break;

      default:
      break;
    }
  }
}

uint32_t makeRandomPasskey(void)
{
	CMU_ClockEnable(cmuClock_TRNG0, true);							// enable clock to TRNG
	TRNG0->CONTROL = 1; 											// enable TRNG
	uint32_t n = TRNG0->FIFO; 										// get 6-digit TRN
	n = (n % 900000) + 100000;
	return(n);
}

uint32_t readPasskey(void)											// read 6-digit passkey
{
	uint32_t n = 0;
	for (uint8_t i=0; i<6; i++) {
		do {c = getchar();} while (c == 255);						// wait for key pressed
		putchar(c);													// echo char to terminal
		n = n*10 + (c - '0');
	}
	return(n);
}
